/* リスト3.4 ARマーカの上にMQO形式の3Dモデルを表示するサンプル */

import javax.media.opengl.*;  // JavaのOpenGL（JOGL)の機能を使うために必要
import processing.opengl.*;   // ProcessingのOpenGLの機能を使うために必要
import jp.nyatla.nyar4psg.*;  // NyAR4psgを使うために必要
import processing.video.*;    // ビデオ入力を扱うので必要
import jp.nyatla.kGLModel.*;                 // MQOローダを使うために必要
import jp.nyatla.kGLModel.contentprovider.*; // MQOローダを使うために必要

Capture cam;    // キャプチャ
MultiMarker ar; // ARマーカに関する処理をするオブジェクト
int id;         // マーカに割り当てられるID番号
KGLModelData     model_data;        // モデルデータ
ContentProvider  content_provider;  // ファイル読み込み

void setup() {
  size(640, 480, OPENGL);  
  cam = new Capture(this, width, height); // キャプチャの準備
  ar = new MultiMarker(this, width, height, "camera_para.dat", NyAR4PsgConfig.CONFIG_PSG );
  id = ar.addARMarker("patt.hiro", 80); // パターンファイルとマーカサイズの登録

  // MQOモデルの準備
  content_provider = new LocalContentProvider(this,dataPath("ninja.mqo"));
  PGraphicsOpenGL pgl = (PGraphicsOpenGL) g;
  GL gl = pgl.beginGL();
  model_data = KGLModelData.createGLModelPs( 
                this, gl, null, 
                this.content_provider,1.0,
                KGLExtensionCheck.IsExtensionSupported( gl,
                "GL_ARB_vertex_buffer_object"),
                true );
  pgl.endGL();
}

void draw() {
  if (cam.available()==false) return;
  cam.read();             // カメラ画像の読み込み
  background(0);          // 画面の初期化
  ar.drawBackground(cam); // 背景画像の描画
  ar.detect(cam);         // マーカ認識
  
  // マーカが検出されたら
  if ( ar.isExistMarker(id) ) {
    ar.beginTransform(id);    
      PGraphicsOpenGL pgl = (PGraphicsOpenGL) g;
      GL gl = pgl.beginGL();
        // Processingの行列をOpenGLで使えるようにする 
        gl.glMatrixMode( gl.GL_PROJECTION );
        gl.glLoadMatrixf( convMatrixPsg2GL(((PGraphics3D)g).projection), 0 );
        gl.glMatrixMode( gl.GL_MODELVIEW );
        gl.glLoadIdentity();  
        gl.glScalef( 1.0, -1.0, 1.0 );
        gl.glMultMatrixf( convMatrixPsg2GL(((PGraphics3D)g).modelview), 0 );
      
        // テクスチャとポリゴンの描画に関する設定
        gl.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_MODULATE);
        gl.glEnable(GL.GL_CULL_FACE);
        gl.glCullFace(GL.GL_FRONT);
      
        // 光源の設定        
        mySetLight(gl, 0, -100, 100);
        gl.glRotatef(90, 1, 0, 0); // 立たせるために回転
        gl.glRotatef(180,0,1,0);   // 正面を向かせるために回転
        model_data.draw() ;
      pgl.endGL();
    ar.endTransform();    
  }
}

// 光源の設定を行う関数
void mySetLight(GL gl, float x, float y, float z) {
  float[] light_diffuse  = { 0.2, 0.2, 0.2, 1.0 };  // 拡散反射光
  float[] light_specular = { 0.3, 0.3, 0.3, 1.0 };  // 鏡面反射光
  float[] light_ambient  = { 0.3, 0.3, 0.3, 0.1 };  // 環境光
  float[] light_position = { x, y, z, 0.0 };        // 位置と種類

  gl.glLightfv( gl.GL_LIGHT0, gl.GL_DIFFUSE,  light_diffuse, 0); // 拡散反射光
  gl.glLightfv( gl.GL_LIGHT0, gl.GL_SPECULAR, light_specular,0); // 鏡面反射光
  gl.glLightfv( gl.GL_LIGHT0, gl.GL_AMBIENT,  light_ambient, 0); // 環境光
  gl.glLightfv( gl.GL_LIGHT0, gl.GL_POSITION, light_position,0); // 位置と種類
  gl.glShadeModel( gl.GL_SMOOTH ); // シェーディングの種類の設定
  gl.glEnable( gl.GL_LIGHT0 );     // 光源0の有効化
  gl.glEnable( gl.GL_LIGHTING );   // ライティングの有効化 
}

// Processingの行列をOpenGLの行列に変換する関数
float[] convMatrixPsg2GL( PMatrix3D mat_psg ) {
  float[] mat_gl = new float [16];
  PMatrix3D mat = mat_psg.get();
  mat.transpose();
  mat.get(mat_gl);
  return mat_gl;
}
