/* リスト3.17 ユーザごとに異なる色で塗るサンプル */

import SimpleOpenNI.*;

SimpleOpenNI context;
color[] userColors = { color(255,0,0), color(0,255,0), color(0,0,255),
color(255,255,0), color(255,0,255), color(0,255,255) };

void setup() {
  context = new SimpleOpenNI(this);
  context.setMirror(false);                          // ミラー反転を無効にする
  context.enableDepth();                             // デプス画像を有効にする
  context.enableUser(SimpleOpenNI.SKEL_PROFILE_ALL); // ユーザトラッキングを有効にする
  size(context.depthWidth(), context.depthHeight());
}

void draw() {
  // カメラの更新とデプス画像の描画
  context.update();
  image(context.depthImage(), 0, 0);

  // ユーザが1人以上いるなら，ユーザマップを取得
  int[] userMap = null;
  int userCount = context.getNumberOfUsers();
  if (userCount > 0) {
    userMap = context.getUsersPixels(SimpleOpenNI.USERS_ALL);
  }

  // ユーザマップの情報を元に，ユーザがいるピクセルに色を塗る
  loadPixels();
  for (int y=0; y<context.depthHeight(); y++) {
    for (int x=0; x<context.depthWidth(); x++) {
      int index = x + y * context.depthWidth();
      // ユーザマップを読み，値が0より大きければユーザが居るピクセルだと判定
      if ( userMap != null && userMap[index] > 0 ) {
        int colorIndex = userMap[index] % userColors.length;  // ユーザ色を決定
        pixels[index] = userColors[colorIndex];    // ピクセル値をユーザ色にする
      }
    } 
  }
  updatePixels();
}

