/* リスト4.1 距離センサの特性を調べるためのプログラム */

import processing.serial.*;  // シリアル通信ライブラリ
import cc.arduino.*;         // Arduinoライブラリ

Arduino arduino;
int sensorPin = 0; // センサが接続されているアナログピンの番号

int d1 = 400;  // 距離１[mm]
int d2 = 1200; // 距離２[mm]
float a = 0; // 距離計算式の定数
float b = 0; // 距離計算式の定数

float value = 0; // 入力値
float v1 = 0;  // 距離１のときの入力値（キー入力時に取得）
float v2 = 0;  // 距離２のときの入力値（キー入力時に取得）

// 移動平均に使う変数
float[] buf = null;  // 移動平均用のリングバッファ
int N = 10;          // リングバッファのサイズ
int k = 0;           // リングバッファのインデックス

void setup() {
  // Arduinoの接続設定（ポート番号は自分の環境に合わせてください）
  arduino = new Arduino(this, Arduino.list()[0], 57600);
  buf = new float[N]; // 移動平均用のバッファを準備する
}

void draw() {  
  // 入力値に対して移動平均を行う
  buf[k] = arduino.analogRead(sensorPin);
  k = (k+1)%N;
  float sum = 0;
  for (int i=0; i<N; i++) {
    sum += buf[i];
  }
  value = sum/N;

  // 距離の計算
  float distance = a / (value + b);
  println("a=" + a);
  println("b=" + b);
  println("distance=" + distance);
}

// キー入力時の操作
void keyPressed() { 
  if (key=='1')      v1 = value;  // 「1」を押すと距離１のときの入力値として保存
  else if (key=='2') v2 = value;  // 「2」を押すと距離２のときの入力値として保存
  
  // 定数の計算
  b = -(d1*v1 - d2*v2) / (d1 - d2);
  a = d1 * (v1 + b);
}
